Package: vulcan
Type: Package
Title: VirtUaL ChIP-Seq data Analysis using Networks
Version: 1.35.0
Author: Federico M. Giorgi, Andrew N. Holding, Florian Markowetz
Maintainer: Federico M. Giorgi <federico.giorgi@gmail.com>
Description: Vulcan (VirtUaL ChIP-Seq Analysis through Networks) is a
        package that interrogates gene regulatory networks to infer
        cofactors significantly enriched in a differential binding
        signature coming from ChIP-Seq data. In order to do so, our
        package combines strategies from different BioConductor
        packages: DESeq for data normalization, ChIPpeakAnno and
        DiffBind for annotation and definition of ChIP-Seq genomic
        peaks, csaw to define optimal peak width and viper for applying
        a regulatory network over a differential binding signature.
License: LGPL-3
biocViews: SystemsBiology, NetworkEnrichment, GeneExpression, ChIPSeq
NeedsCompilation: no
Suggests: vulcandata
Depends: R (>= 4.0), ChIPpeakAnno,TxDb.Hsapiens.UCSC.hg19.knownGene,
        zoo, GenomicRanges, S4Vectors, viper, DiffBind, locfit
Imports: wordcloud, csaw, gplots, stats, utils, caTools, graphics,
        DESeq2, Biobase
Encoding: UTF-8
RoxygenNote: 7.1.1
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev libjpeg-dev
        liblzma-dev libpng-dev libuv1-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:46:32 UTC
RemoteUrl: https://github.com/bioc/vulcan
RemoteRef: HEAD
RemoteSha: c42f6e949d3182a7e7e35e0bf27b67f98174f5bb
Packaged: 2026-05-29 09:34:02 UTC; root
Built: R 4.6.0; ; 2026-05-29 11:18:02 UTC; unix

# Generated by roxygen2: do not edit by hand

export(average_fragment_length)
export(corr2p)
export(densityauc)
export(dpareto)
export(fisherp)
export(gsea)
export(kmgformat)
export(null_gsea)
export(p2corr)
export(p2z)
export(pareto.fit)
export(plot_gsea)
export(ppareto)
export(rea)
export(slice)
export(stouffer)
export(textplot2)
export(val2col)
export(vulcan)
export(vulcan.annotate)
export(vulcan.import)
export(vulcan.normalize)
export(vulcan.pathways)
export(wstouffer)
export(z2p)

import(zoo)
import(ChIPpeakAnno)
import(locfit)
import(TxDb.Hsapiens.UCSC.hg19.knownGene)
importFrom('GenomicRanges','GRanges')
importFrom('S4Vectors','Rle')
importFrom('gplots','colorpanel')
importFrom('caTools','runmean')
importFrom('csaw','correlateReads')
importFrom('wordcloud','wordlayout')
importFrom('DESeq2','DESeqDataSetFromMatrix','varianceStabilizingTransformation')
importFrom('Biobase','exprs')
importFrom('viper','rowTtest','msviper','msviperAnnot','ttestNull')
importFrom('DiffBind','dba','dba.count')
importFrom('graphics', 'abline', 'grid', 'layout', 'legend', 'lines',
    'par', 'plot', 'points', 'rect', 'text')
importFrom('stats', 'ks.test', 'pchisq', 'pnorm', 'pt', 'qnorm', 'qt',
    'rnorm', 'setNames', 'quantile', 'var',
    'predict', 'smooth.spline')
importFrom('utils', 'read.csv', 'setTxtProgressBar', 'txtProgressBar', 'relist')

# Building manuals and namespace using roxygen2
library(devtools)
document()


# Remove .Rhistory
unlink(".Rhistory")
unlink("R/.Rhistory")


# # # Force the formatR style
# library(formatR)
# getOption("width",80)
# for(rfile in dir("R",full.names=TRUE)){
#     tidy_source(source=rfile,file=rfile,width.cutoff=40,indent=4)
# }


cat("\nimport(zoo)\n",file="NAMESPACE",append=TRUE)
cat("import(ChIPpeakAnno)\n",file="NAMESPACE",append=TRUE)
cat("import(locfit)\n",file="NAMESPACE",append=TRUE)
cat("import(TxDb.Hsapiens.UCSC.hg19.knownGene)\n",file="NAMESPACE",append=TRUE)


cat("importFrom('GenomicRanges','GRanges')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('S4Vectors','Rle')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('gplots','colorpanel')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('caTools','runmean')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('csaw','correlateReads')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('wordcloud','wordlayout')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('DESeq2','DESeqDataSetFromMatrix','varianceStabilizingTransformation')\n",
    file="NAMESPACE",append=TRUE)
cat("importFrom('Biobase','exprs')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('viper','rowTtest','msviper','msviperAnnot','ttestNull')\n",file="NAMESPACE",append=TRUE)
cat("importFrom('DiffBind','dba','dba.count')\n",file="NAMESPACE",append=TRUE)




cat("importFrom('graphics', 'abline', 'grid', 'layout', 'legend', 'lines',\n    'par', 'plot', 'points', 'rect', 'text')\n",
    file="NAMESPACE",append=TRUE)
cat("importFrom('stats', 'ks.test', 'pchisq', 'pnorm', 'pt', 'qnorm', 'qt',\n    'rnorm', 'setNames', 'quantile', 'var',\n    'predict', 'smooth.spline')\n",
    file="NAMESPACE",append=TRUE)
cat("importFrom('utils', 'read.csv', 'setTxtProgressBar', 'txtProgressBar', 'relist')\n",
    file="NAMESPACE",append=TRUE)

build()

library(BiocCheck)
#BiocCheck("/Users/giorgi01/Dropbox/rstudio/vulcanpackage/vulcan")
BiocCheck(getwd())

#' GSEA
#'
#' This function performs Gene Set Enrichment Analysis
#' @param reflist named vector of reference scores
#' @param set element set
#' @param method one of 'permutation' or 'pareto'
#' @param w exponent used to raise the supplied scores. Default is 1 (original
#' scores unchanged)
#' @param np Number of permutations (Default: 1000)
#' @param gsea_null a GSEA null distribution (Optional)
#' @return A GSEA object. Basically a list of s components:
#' \describe{
#' \item{ES}{The enrichment score}
#' \item{NES}{The normalized enrichment socre}
#' \item{ledge}{The items in the leading edge}
#' \item{p.value}{The permutation-based p-value}
#' }
#' @examples
#' reflist<-setNames(-sort(rnorm(1000)),paste0('gene',1:1000))
#' set<-paste0('gene',sample(1:200,50))
#' obj<-gsea(reflist,set,method='pareto',np=1000)
#' obj$p.value
#' @export
gsea <- function(reflist,
                set,
                method=c("permutation","pareto"),
                np=1000,
                w=1,
                gsea_null=NULL) {


    # Get elements in set that are in the ref list
    set <- intersect(names(reflist), set)

    # Sort the reference list
    # Get the list order, from higher (1)to smaller (n)
    ix <- order(reflist, decreasing=TRUE)
    reflist <- reflist[ix] # Reorder the reference list

    # Initialize variables for running sum
    es <- 0
    nes <- 0
    p.value <- 1

    # Identify indexes of set within the sorted reference list
    inSet <- rep(0, length(reflist))
    inSet[which(names(reflist) %in% set)] <- 1

    ### Compute Enrichment Score
    # Compute running sum for hits
    hits<-abs(reflist*inSet) # Get the values for the elements in the set
    hits<-hits^w # Raise this score to the power of w
    score_hit <- cumsum(hits) # Cumulative sum of hits' scores
    # The cumulative sum is divided by the final  sum value
    score_hit <- score_hit / score_hit[length(score_hit)]

    # Compute running sum for non-hits
    score_miss <- cumsum(1-inSet)
    score_miss <- score_miss/score_miss[length(score_miss)]

    # The Running Score is the difference between the two scores! Hits - nonhits
    running_score <- score_hit - score_miss

    # Safety measure, in the case the random genes have all a weight of 0
    if(all(is.na(running_score))){
        running_score<-rep(0,length(running_score))
    }

    # The ES is actually the minimum or maximum Running Scores
    if(abs(max(running_score))>abs(min(running_score))){
        es<-max(running_score)
    } else {
        es<-min(running_score)
    }


    ### Identify leading edge
    # Create a vector of 0s long as the reference list
    ledge_indeces <- rep(0, length(running_score))
    # Case 1: negative ES
    if (es<0){
        peak <- which(running_score==min(running_score))[1]
        # Leading edge is stuff AFTER the peak point (ES is negative)
        ledge_indeces[peak:length(ledge_indeces)] <- 1
        ledge_indeces <- which(ledge_indeces == 1)
        ledge_names <- names(reflist[ledge_indeces])
    } else{ # Case 2: positive ES
        peak <- which(running_score==max(running_score)) # Define the peak point
        # Leading edge is stuff BEFORE the peak point (ES is positive)
        ledge_indeces[1:peak] <- 1
        ledge_indeces <- which(ledge_indeces == 1)
        ledge_names <- names(reflist[ledge_indeces])
    }





    ### Compute p-value by permutation
    if(is.null(gsea_null)){
        null_es<-null_gsea(set=set,reflist=reflist,np=np,w=w)
    } else{
        ### If a null list is provided, use it
        if(class(gsea_null)=="gsea_nullist"){
            null_es<-gsea_null[as.character(length(set))][[1]]
        }else{
            null_es<-gsea_null
        }
    }
    if (es<0){
        p.value <- sum(null_es<=es)/length(null_es)
    } else {
        p.value <- sum(null_es>=es)/length(null_es)
    }
    #    }


    # If we are in the tail, the p-value can be calculated in two ways
    if(is.na(p.value) || p.value<0.05) {
        if(p.value==0){
            p.value <- 1/np
        }
        if (method=="pareto"){
            # Extract the absolute null ESs above the 95th percentile
            q95<-as.numeric(quantile(abs(null_es),0.95))
            fit<-pareto.fit(abs(null_es),threshold=q95)
            newp.value<-ppareto(abs(es), threshold=q95,
                                exponent=fit$exponent, lower.tail=FALSE)/20
        # If Pareto cannot infer small ESs take the permutation p-value
            if(is.na(newp.value)){
                newp.value<-p.value
            }
            p.value<-newp.value
        }
    }

    # Calculate the normalized enrichment score
    nes<-p2z(p.value)*sign(es)

    gsea.obj<-list(
        es=es,
        nes=nes,
        p.value=p.value,
        ledge=ledge_names,
        running_score=running_score,
        set=set,
        reflist=reflist,
        inSet=inSet
    )
    class(gsea.obj)<-"gsea"
    return(gsea.obj)
}


#' Calculate Null Distribution for GSEA
#'
#' This function generates a GSEA null distribution from
#'
#' @param set A vector containing gene names.
#' @param reflist A named vector containing the weights of the entire signature
#' @param np Number of permutations (Default: 1000)
#' @param w exponent used to raise the supplied scores. Default is 1 (original
#' scores unchanged)
#' @return A vector of null scores appropriate for the set/reflist combination
#' provided
#' @examples
#' reflist<-setNames(-sort(rnorm(26)),LETTERS)
#' set<-c('A','B','D','F')
#' nulldist<-null_gsea(set,reflist)
#' nulldist[1:10]
#' @export
null_gsea<-function(set,reflist,w=1,np=1000){
    gsea_null <- rep(0, np)
    gsea_null <- sapply(1:np, function(i) {
        # Identify indexes of set within the sorted reference list
        inSet <- rep(0, length(reflist))
        inSet[which(names(reflist) %in% set)] <- 1

        # By sampling the order of the set elements, we get the real permutation
        null_inSet <- inSet[sample(1:length(inSet))]

        # Same as before, cumulative sums of hits and nonhits
        null_hit<-abs(reflist*null_inSet)
        null_hit<-null_hit^w
        null_hit <- cumsum(null_hit)
        null_hit <- null_hit/null_hit[length(null_hit)]
        null_miss <- cumsum(1-null_inSet)
        null_miss <- null_miss/null_miss[length(null_miss)]
        # And dependending on the cumulative sums, null running sum and
        # null enrichment score
        null_running_score <- null_hit - null_miss

        # The ES is just he maximum or the minimum
        if(abs(max(null_running_score))>abs(min(null_running_score))){
            null_es<-max(null_running_score)
        } else {
            null_es<-min(null_running_score)
        }
        return(null_es)
    })
    class(gsea_null)<-"gsea_null"
    return(gsea_null)
}
#' Plot GSEA results
#'
#' This function generates a GSEA plot from a gsea object
#'
#' @param gsea.obj GSEA object produced by the \code{gsea} function
#' @param twoColors the two colors to use for positive[1] and negative[2]
#' enrichment scores
#' @param colBarcode The color of the barcode
#' @param plotNames Logical. Should the set names be plotted?
#' @param title String to be plotted above the Running Enrichment Score
#' @param bottomYlabel String for the label
#' @param bottomYtitle String for the title of the bottom part of the plot
#' @param ext_nes Provide a NES from an external calculation
#' @param omit_middle If TRUE, will not plot the running score
#' (FALSE by default)
#' @return Nothing, a plot is generated in the default output device
#' @examples
#' reflist<-setNames(-sort(rnorm(26)),LETTERS)
#' set<-c('A','B','D','F')
#' obj<-gsea(reflist,set,method='pareto')
#' plot_gsea(obj)
#' @export
plot_gsea <- function(gsea.obj, twoColors = c("red",
                                            "blue"),
                    plotNames = FALSE, colBarcode = "black",
                    title = "Running Enrichment Score",
                    bottomYtitle = "List Values",
                    bottomYlabel = "Signature values",
                    ext_nes = NULL, omit_middle = FALSE) {
    # Extract parameters from the gsea object
    es <- gsea.obj$es
    nes <- gsea.obj$nes
    p.value <- gsea.obj$p.value
    ledge <- gsea.obj$ledge
    running_score <- gsea.obj$running_score
    set <- gsea.obj$set
    reflist <- gsea.obj$reflist
    inSet <- gsea.obj$inSet

    # Define plot borders? Who wrote the
    # original code has a non-euclidean mind
    min.RES <- min(running_score)
    max.RES <- max(running_score)
    delta <- (max.RES - min.RES) * 0.5
    min.plot <- min.RES
    max.plot <- max.RES
    max.corr <- max(reflist)
    min.corr <- min(reflist)
    corr0.line <- (-min.corr/(max.corr -
                                min.corr)) * 1.25 * delta + min.plot

    if (es < 0) {
        l.ledge.ref.plot <- length(reflist) -
            length(ledge)
    } else {
        l.ledge.ref.plot <- length(ledge)
    }

    # Define colors (red is positive nes,
    # blue is negative nes)
    if (nes > 0) {
        col.f <- twoColors[1]
    } else {
        col.f <- twoColors[2]
    }
    N <- length(reflist)
    ind <- 1:N


    ### Define layout, let's end this
    ### destructive putting everything in a
    ### single window
    if (omit_middle) {
        layoutMatrix <- rbind(1, 2)
        layout(layoutMatrix, heights = c(1,
                                        2))
    } else {
        layoutMatrix <- rbind(1, 2, 3)
        layout(layoutMatrix, heights = c(1,
                                        4, 2))
    }


    # layout.show(n=3)

    ### PLOT 1: barcode-like enrichment tags
    par(mar = c(0, 4.1, 2, 2.1))
    plot(0, col = "white", xlim = c(1, N),
        ylim = c(0, 10), xaxt = "n", yaxt = "n",
        type = "n", frame.plot = FALSE, xlab = "",
        ylab = "", xaxs = "r", yaxs = "r",
        main = paste("Number of elements: ",
                    N, " (in full list), ", length(set),
                    " (in element set)", sep = "",
                    collapse = ""))
    for (position in 1:N) {
        if (inSet[position] == 1) {
            if (N < 50 & length(set) <= 10) {
                rect(xleft = position - 0.2,
                    ybottom = 0, xright = position +
                        0.2, ytop = 10, col = colBarcode,
                    border = NA)
            } else {
                abline(v = position, lwd = 2,
                    col = colBarcode)
            }
            if (plotNames) {
                text(labels = names(reflist[position]),
                    x = position - 0.2, y = 0,
                    srt = 90, offset = 0, pos = 4,
                    font = 2)
            }
        }
    }

    ### PLOT 2: The running sum plot
    if (!omit_middle) {
        par(mar = c(2, 4.1, 2, 2.1))
        plot(ind, running_score, sub = "",
            xlab = "", ylab = "Enrichment Score",
            xlim = c(1, N), ylim = c(min.plot,
                                    max.plot), type = "l", pch = 20,
            lwd = 4, cex = 1, col = col.f,
            xaxs = "r", yaxs = "r", main = title)
        grid(col = "dark grey", lty = 2)

        # This is important: zero running score
        # line
        lines(c(1, N), c(0, 0), lwd = 1,
            lty = 1, cex = 1)

        # This is also important: it's the max
        # enrichment vertical line (aka LEADING
        # EDGE)
        lines(c(l.ledge.ref.plot, l.ledge.ref.plot),
            c(min.plot, max.plot), lwd = 1,
            lty = 1, cex = 1)

        if (es >= 0) {
            legend_position <- "topright"
        } else {
            legend_position <- "topleft"
        }

        # If an external NES is not provided, the
        # standard GSEA one is shown
        if (is.null(ext_nes)) {
            legend(legend_position,
                legend = c(paste("ES = ",
                                    signif(es, 3), sep = ""),
                            paste("NES = ", signif(nes,
                                                    3), sep = ""),
                            paste("p-value = ",
                                    signif(p.value, 3), sep = "")),
                bg = "white")
        } else {
            legend(legend_position,
                legend = c(paste("NES = ",
                                    signif(ext_nes, 3), sep = ""),
                            paste("p-value = ",
                                    signif(z2p(ext_nes),
                                        3), sep = "")), bg = "white")
        }
    }


    ### Plot3: weight values
    if (omit_middle) {
        bottomMain <- title
    } else {
        bottomMain <- bottomYtitle
    }

    par(mar = c(2, 4.1, 2, 2.1))
    plot(ind, reflist, type = "l", pch = 20,
        lwd = 3, xlim = c(1, N), cex = 1,
        col = 1, xaxs = "r", yaxs = "r",
        main = bottomMain, ylab = bottomYlabel,
        cex.axis = 0.8)
    grid(col = "dark grey", lty = 2)
    # zero correlation horizontal line
    lines(c(1, N), c(corr0.line, corr0.line),
        lwd = 1, lty = 1, cex = 1, col = 1)

    if (omit_middle) {
        # If an external NES is not provided, the
        # standard GSEA one is shown
        if (is.null(ext_nes)) {
            legend("top",
                legend = c(paste("ES = ",
                                    signif(es, 3), sep = ""),
                            paste("NES = ",
                                    signif(nes,
                                        3), sep = ""),
                            paste("p-value = ",
                                    signif(p.value, 3), sep = "")),
                bg = "white")
        } else {
            legend("top",
                legend = c(paste("NES = ",
                                    signif(ext_nes, 3), sep = ""),
                            paste("p-value = ", signif(z2p(ext_nes),
                                                        3), sep = "")),
                bg = "white")
        }
    }

}



#' Probability density of Pareto distributions
#'
#' Gives NA on values below the threshold
#'
#' @param x Data vector of log probability densities
#' @param threshold numeric value to define the start of the tail
#' @param exponent the exponent obtained from the pareto.fit function
#' @param log logical, should the values be log-transformed?
#' @return Vector of (log) probability densities
#' @examples
#' set.seed(1)
#' x<-abs(rnorm(1000))
#' n<-length(x)
#' exponent<-1+n/sum(log(x))
#' dp<-dpareto(x,exponent=exponent)
#' plot(dp)
#' @export
dpareto <- function(x, threshold = 1, exponent,
                    log = FALSE) {
    # Avoid doing limited-precision
    # arithmetic followed by logs if we want
    # the log!
    if (!log) {
        prefactor <- (exponent - 1)/threshold
        f <- function(x) {
            prefactor * (x/threshold)^(-exponent)
        }
    } else {
        prefactor.log <- log(exponent - 1) -
            log(threshold)
        f <- function(x) {
            prefactor.log - exponent * (log(x) -
                                            log(threshold))
        }
    }
    d <- ifelse(x < threshold, NA, f(x))
    return(d)
}

#' Cumulative distribution function of the Pareto distributions
#' '
#' Gives NA on values < threshold
#' @param x Data vector, lower threshold, scaling exponent, usual flags
#' @param threshold numeric value to define the start of the tail
#' @param exponent the exponent obtained from the pareto.fit function
#' @param lower.tail logical. If the lower tail of the distribution should be
#' considered. Default is TRUE
#' @return Vector of (log) probabilities
#' @examples
#' # Estimate the tail of a pospulation normally distributed
#' set.seed(1)
#' x<-rnorm(1000)
#' q95<-as.numeric(quantile(abs(x),0.95))
#' fit<-pareto.fit(abs(x),threshold=q95)
#' # We can infer the pvalue of a value very much right to the tail of the
#' # distribution
#' value<-5
#' pvalue<-ppareto(value, threshold=q95, exponent=fit$exponent,
#' lower.tail=FALSE)/20
#' plot(density(abs(x)),xlim=c(0,value+0.3),main='Pareto fit')
#' arrows(value,0.2,value,0)
#' text(value,0.2,labels=paste0('p=',signif(pvalue,2)))
#' @export
ppareto <- function(x, threshold = 1, exponent,
                    lower.tail = TRUE) {
    if (!lower.tail) {
        f <- function(x) {
            (x/threshold)^(1 - exponent)
        }
    }
    if (lower.tail) {
        f <- function(x) {
            1 - (x/threshold)^(1 - exponent)
        }
    }
    p <- ifelse(x < threshold, NA, f(x))
    return(p)
}

#' Estimate parameters of Pareto distribution
#'
#' A wrapper for functions implementing actual methods
#' @param data data vector, lower threshold (or 'find', indicating it should be
#' found from the data), method (likelihood or regression, defaulting to former)
#' @param threshold numeric value to define the start of the tail
#' @return List indicating type of distribution ('pareto'), parameters,
#' information about fit (depending on method), OR a warning and NA if method
#' is not recognized
#' @examples
#' # Estimate the tail of a pospulation normally distributed
#' set.seed(1)
#' x<-rnorm(1000)
#' q95<-as.numeric(quantile(abs(x),0.95))
#' fit<-pareto.fit(abs(x),threshold=q95)
#' # We can infer the pvalue of a value very much right to the tail of the
#' # distribution
#' value<-5
#' pvalue<-ppareto(value, threshold=q95, exponent=fit$exponent,
#' lower.tail=FALSE)/20
#' plot(density(abs(x)),xlim=c(0,value+0.3),main='Pareto fit')
#' arrows(value,0.2,value,0)
#' text(value,0.2,labels=paste0('p=',signif(pvalue,2)))
#' @export
pareto.fit <- function(data, threshold) {
    data <- data[data >= threshold]
    n <- length(data)
    x <- data/threshold
    alpha <- 1 + n/sum(log(x))
    # Calculate Log-Likelihood
    loglike <- sum(dpareto(data, threshold = threshold,
                        exponent = alpha, log = TRUE))
    # KS distance
    newdata <- data[data >= threshold]
    d <- suppressWarnings(ks.test(newdata,
                                ppareto, threshold = threshold,
                                exponent = alpha))
    ks.dist <- as.vector(d$statistic)
    fit <- list(type = "pareto", exponent = alpha,
                xmin = threshold, loglike = loglike,
                ks.dist = ks.dist, samples.over.threshold = n)
    return(fit)
}

#' Function to import BAM files
#'
#' This function coalesces and annotates a set of BAM files into peak-centered
#' data
#'
#' @param sheetfile path to a csv annotation file containing sample information
#' and BAM location
#' @param intervals size of the peaks. If NULL (default) it is inferred from
#' the average fragment length observed in the dataset
#'
#' @return A list of components:
#' \describe{
#' \item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
#' columns}
#' \item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
#' columns}
#' \item{samples}{A vector of sample names and conditions}
#' }
#' @examples
#' library(vulcandata)
#' # Generate an annotation file from the dummy ChIP-Seq dataset
#' vfile<-tempfile()
#' vulcandata::vulcansheet(vfile)
#' # Import BAM and BED information into a list object
#' # vobj<-vulcan.import(vfile)
#' # This vobj is identical to the object returned by
#' # vulcandata::vulcanexample()
#' unlink(vfile)
#' @export
vulcan.import <- function(sheetfile, intervals = NULL) {
    # Check the dataset
    sheet <- read.csv(sheetfile, as.is = TRUE)

    # Generate a DiffBind object
    dbobj <- DiffBind::dba(sampleSheet = sheetfile)
    message("Sheet loaded. You have ", nrow(sheet),
            " samples and ", length(unique(sheet$Condition)),
            " conditions")

    # Select the interval size automatically
    # (if not provided by the user)
    if (is.null(intervals)) {
        # List of bam files
        bam.files <- sheet[, "bamReads"]
        intervals <- average_fragment_length(bam.files, plot = FALSE) * 2
        message("Peak size automatically detected as ",
                intervals, "nt")
    }

    # Count reads in binding sites intervals
    dbcounts <- DiffBind::dba.count(dbobj, summits = intervals)

    # Extract counts from the dbacount object
    listcounts <- dbcounts$peaks
    names(listcounts) <- dbcounts$samples[,1]

    # Prepare RPKM matrix
    first <- listcounts[[1]]
    rawmat <- matrix(NA, nrow = nrow(first), ncol = length(listcounts) + 3)
    colnames(rawmat) <- c("Chr", "Start", "End", names(listcounts))
    rownames(rawmat) <- 1:nrow(rawmat)
    rawmat <- as.data.frame(rawmat)
    rawmat[, 1] <- as.character(first[, 1])
    rawmat[, 2] <- as.integer(first[, 2])
    rawmat[, 3] <- as.integer(first[, 3])
    for (i in seq_len(length(listcounts))) {
        rawmat[, names(listcounts)[i]] <- as.numeric(listcounts[[i]]$RPKM)
    }
    peakrpkms <- rawmat

    # Prepare Count matrix
    first <- listcounts[[1]]
    rawmat <- matrix(NA, nrow = nrow(first), ncol = length(listcounts) + 3)
    colnames(rawmat) <- c("Chr", "Start", "End", names(listcounts))
    rownames(rawmat) <- 1:nrow(rawmat)
    rawmat <- as.data.frame(rawmat)
    rawmat[, 1] <- as.character(first[, 1])
    rawmat[, 2] <- as.integer(first[, 2])
    rawmat[, 3] <- as.integer(first[, 3])
    for (i in seq_len(length(listcounts))) {
        rawmat[, names(listcounts)[i]] <- as.integer(listcounts[[i]]$Reads)
    }
    peakcounts <- rawmat


    # Create an annotation structure
    samples <- list()
    conditions <- unique(sheet$Condition)
    for (cond in conditions) {
        heresamples <- sheet$SampleID[sheet$Condition == cond]
        samples[[cond]] <- heresamples
    }

    # Return output
    vobj <- list(peakcounts = peakcounts, samples = samples,
                peakrpkms = peakrpkms)
    return(vobj)
}

#' Function to annotate peaks for VULCAN analysis
#'
#' This function coalesces and annotates a set of BAM files into peak-centered
#' data. It implements the ChIPPeakANno methods, with specific choices dealing
#' with defining the genomic area around the promoter and which peaks to
#' include.
#'
#' @param vobj A list of peakcounts, samples and peakrpkms (i.e. the output of
#' the function vulcan.import)
#' @param method Method to deal with multiple peaks found within gene promoter
#' boundaries. One of sum (default), closest, strongest, topvar, farthest or
#' lowvar. This will affect only genes with multiple possible peaks. When a
#' single peak can be mapped to the promoter region of the gene, that peak
#' abundance will be considered as the gene promoter's occupancy.
#' \describe{
#' \item{sum}{when multiple peaks are found, sum their contributions}
#' \item{closest}{when multiple peaks are found, keep only the closest to the
#' TSS as the representative one}
#' \item{strongest}{when multiple peaks are found, keep the strongest as the
#' representative one}
#' \item{farthest}{when multiple peaks are found, keep only the closest to the
#' TSS as the representative one}
#' \item{topvar}{when multiple peaks are found, keep the most varying as the
#' representative one}
#' \item{lowvar}{when multiple peaks are found, keep the least varying as the
#' representative one}
#' }
#' @param lborder Boundary for peak annotation (in nucleotides) upstream of
#' the Transcription starting site (default: -10000)
#' @param rborder Boundary for peak annotation (in nucleotides) downstream of
#' the Transcription starting site (default: 10000)
#' @param TxDb TxDb annotation object containing the knownGene track. If NULL
#' (the default), TxDb.Hsapiens.UCSC.hg19.knownGene is loaded
#' @return A list of components:
#' \describe{
#' \item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
#' columns}
#' \item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
#' columns}
#' \item{rawcounts}{A matrix of raw gene counts, genes as rows, samples as
#' columns. The counts are associated to the promoter region of the gene}
#' \item{rpkms}{A matrix of RPKMs, genes as rows, samples as
#' columns. The RPKMs are associated to the promoter region of the gene}
#' \item{samples}{A vector of sample names and conditions}
#' }
#' @examples
#' library(vulcandata)
#' vobj<-vulcandata::vulcanexample()
#' vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
#' @export
vulcan.annotate <- function(vobj, lborder = -10000,
                            rborder = 10000, method = c("closest",
                                                        "strongest",
                                                        "sum",
                                                        "topvar",
                                                        "farthest",
                                                        "lowvar"),
                            TxDb=NULL) {

    if(!is.null(TxDb)){
        # Annotate (any genome)
        annotation <- toGRanges(TxDb,
                                feature = "gene")

    } else {
        # Annotate (hg19)
        annotation <- toGRanges(TxDb.Hsapiens.UCSC.hg19.knownGene,
                                feature = "gene")
    }


    ##### PROCESS RAW COUNTS
    gr <- GRanges(vobj$peakcounts)
    anno <- annotatePeakInBatch(gr, AnnotationData = annotation,
                                output = "overlapping",
                                FeatureLocForDistance = "TSS",
                                bindingRegion = c(lborder, rborder))

    # Convert to a more handy data frame
    dfanno <- anno
    names(dfanno) <- seq_len(length(dfanno))
    dfanno <- as.data.frame(dfanno)

    # Prepare the output table
    allsamples <- unique(unlist(vobj$samples))
    genes <- unique(dfanno$feature)
    peakspergene <- table(dfanno$feature)
    rawcounts <- matrix(NA, nrow = length(genes),
                        ncol = length(allsamples))
    colnames(rawcounts) <- allsamples
    rownames(rawcounts) <- genes

    # All methods: if a gene has a single
    # peak, you select that
    genesone <- names(peakspergene)[peakspergene ==
                                        1]
    for (gene in genesone) {
        rawcounts[gene, allsamples] <- as.numeric(dfanno[dfanno$feature ==
                                                            gene, allsamples])
    }

    # Other methods: they deal with cases
    # where multiple peaks are found
    genesmore <- names(peakspergene)[peakspergene > 1]
    rawcounts<-dist_calc(method,dfanno,rawcounts,genesmore,allsamples)

    ##### PROCESS RPKMS
    gr <- GRanges(vobj$peakrpkms)
    anno <- annotatePeakInBatch(gr, AnnotationData = annotation,
                                output = "overlapping",
                                FeatureLocForDistance = "TSS",
                                bindingRegion = c(lborder, rborder))

    # Convert to a more handy data frame
    dfanno <- anno
    names(dfanno) <- seq_len(length(dfanno))
    dfanno <- as.data.frame(dfanno)

    # Prepare the output table
    allsamples <- unique(unlist(vobj$samples))
    genes <- unique(dfanno$feature)
    peakspergene <- table(dfanno$feature)
    rpkms <- matrix(NA, nrow = length(genes),
                    ncol = length(allsamples))
    colnames(rpkms) <- allsamples
    rownames(rpkms) <- genes

    # All methods: if a gene has a single
    # peak, you select that
    genesone <- names(peakspergene)[peakspergene == 1]
    for (gene in genesone) {
        rpkms[gene, allsamples] <- as.numeric(
            dfanno[dfanno$feature == gene, allsamples]
        )
    }

    # Other methods: they deal with cases
    # where multiple peaks are found
    genesmore <- names(peakspergene)[peakspergene > 1]
    rpkms<-dist_calc(method,dfanno,rpkms,genesmore,allsamples)


    ### Fix data types as needed
    rawcounts<-matrix(as.numeric(rawcounts),nrow=nrow(rawcounts),
                    dimnames=dimnames(rawcounts))
    rpkms<-matrix(as.numeric(rpkms),nrow=nrow(rpkms),
                dimnames=dimnames(rpkms))

    # Return object
    vobj$rawcounts <- rawcounts
    vobj$rpkms <- rpkms
    return(vobj)
}


dist_calc<-function(method,dfanno,genematrix,genesmore,allsamples){
    # This function structure was strongly suggested
    # by the Bioconductor reviewer
    supportedMethods<-c(
        "closest",
        "strongest",
        "sum",
        "topvar",
        "farthest",
        "lowvar"
    )
    if(!method%in%supportedMethods){
        stop("unsupported method ", method)
    }

    # Method closest: when multiple peaks are
    # found, keep only the closest to the TSS
    # as the representative one

    # Method farthest: when multiple peaks
    # are found, keep only the closest to the
    # TSS as the representative one

    # Method sum: when multiple peaks are
    # found, sum their contributions

    # Method strongest: when multiple peaks
    # are found, keep the strongest as the
    # representative one

    # Method topvar: when multiple peaks are
    # found, keep the most varying as the
    # representative one

    # Method lowvar: when multiple peaks are
    # found, keep the least varying as the
    # representative one


    for (gene in genesmore) {
        subanno <- dfanno[dfanno$feature == gene, ]

        if (method == "closest") {
            closest <- which.min(subanno$distanceToStart)
            genematrix[gene, allsamples] <- as.numeric(subanno[closest,
                                                            allsamples])
        }

        if (method == "farthest") {
            farthest <- which.max(subanno$distanceToStart)
            genematrix[gene, allsamples] <- as.numeric(subanno[farthest,
                                                            allsamples])
        }

        if (method == "sum") {
            sums <- apply(subanno[, allsamples], 2, sum)
            genematrix[gene, allsamples] <- as.numeric(sums)
        }

        if (method == "strongest") {
            sums <- apply(subanno[, allsamples], 1, sum)
            top <- which.max(sums)
            genematrix[gene, allsamples] <- as.numeric(subanno[top,
                                                            allsamples])
        }

        if (method == "topvar") {
            vars <- apply(subanno[, allsamples], 1, var)
            top <- which.max(vars)
            genematrix[gene, allsamples] <- as.numeric(subanno[top,
                                                            allsamples])
        }

        if (method == "lowvar") {
            vars <- apply(subanno[, allsamples], 1, var)
            top <- which.min(vars)
            genematrix[gene, allsamples] <- as.numeric(subanno[top,
                                                            allsamples])
        }

    }
    return(genematrix)
}



#' Function to normalize promoter peak data
#'
#' This function normalizes gene-centered ChIP-Seq data using VST
#'
#' @param vobj a list, the output of the \code{'vulcan.annotate'} function
#'
#' @return A list of components:
#' \describe{
#' \item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
#' columns}
#' \item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
#' columns}
#' \item{rawcounts}{A matrix of raw gene counts, genes as rows, samples as
#' columns. The counts are associated to the promoter region of the gene}
#' \item{rpkms}{A matrix of RPKMs, genes as rows, samples as
#' columns. The RPKMs are associated to the promoter region of the gene}
#' \item{normalized}{A matrix of gene abundances normalized by
#' Variance-Stabilizing Transformation (VST), genes as rows, samples as
#' columns. The abundances are associated to the promoter
#' region of the gene}
#' \item{samples}{A vector of sample names and conditions}
#' }
#' @examples
#' \dontrun{
#' library(vulcandata)
#' vobj<-vulcandata::vulcanexample()
#' vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
#' vobj<-vulcan.normalize(vobj)
#' }
#' @export
vulcan.normalize <- function(vobj) {
    # Extract raw counts from object
    samples <- vobj$samples
    rawcounts <- vobj$rawcounts
    allsamples <- unique(unlist(samples))
    allgenes <- rownames(rawcounts)

    # # Generate a normalized abundance object
    # conditions <- c()
    # for (i in seq_len(length(samples))) {
    #     conditions <- c(conditions, rep(names(samples)[i],
    #                                     length(samples[[i]])))
    # }
    # conditions <- factor(conditions)
    # cds <- DESeq::newCountDataSet(vobj$rawcounts, conditions)
    # cds <- DESeq::estimateSizeFactors(cds)
    # cds <- DESeq::estimateDispersions(cds, fitType = "local")
    # vsd <- DESeq::varianceStabilizingTransformation(cds)
    # normalized <- Biobase::exprs(vsd)
    # rownames(normalized) <- rownames(rawcounts)

    # Accessory functions
    f.rvar.na <- function(x) {
        ave <- as.vector(f.rmean.na(x))
        pos <- which(is.na(x))
        largo <- f.rlength.na(x)
        x[pos] <- rep(ave,ncol(x))[pos]
        (x-ave)^2 %*% rep(1,ncol(x))/(largo-1)
    }
    f.rmean.na <- function(x) {
        largo <- f.rlength.na(x)
        x[is.na(x)] <- 0
        res <- x %*% rep(1,ncol(x)) / largo
        names(res) <- rownames(x)
        res
    }
    f.rlength.na <- function(x) {
        r <- x/x
        r[x==0] <- 1
        r[!is.finite(r)] <- 0
        r %*% rep(1,ncol(r))
    }

    # Normalization based on DESeq2
    set.seed(1)
    coldata<-as.data.frame(colnames(rawcounts))
    coldata[,1]<-as.factor(coldata[,1])
    colnames(coldata)<-"samplename"
    x<-DESeq2::DESeqDataSetFromMatrix(countData=rawcounts,
                              colData=coldata,
                              design=~samplename
    )
    x<-DESeq2::varianceStabilizingTransformation(x,blind=TRUE)
    x<-assay(x)
    #
    tmp <- apply(x, 2, function(x) {
        x <- sort(unique(x))
        x <- cbind(x[1:(length(x)-1)], x[2:length(x)])
        x <- cbind(x[, 1], sqrt(f.rvar.na(x)))
        return(x)
    })
    tmp <- cbind(unlist(lapply(tmp, function(x) x[, 1]), use.names=F), unlist(lapply(tmp, function(x) x[, 2]), use.names=F))
    tmp1 <- stats::smooth.spline(tmp[, 1], tmp[, 2], spar=.5)
    tmp[tmp[, 1]>tmp1$x[which.min(tmp1$y)], 2] <- 0
    tmp1 <- stats::smooth.spline(tmp[, 1], tmp[, 2], spar=.5)
    normalized<-x+rnorm(length(x))*stats::predict(tmp1, x)$y
    #
    vobj$normalized <- normalized
    return(vobj)
}

#' VULCAN - VirtUaL Chipseq data Analysis using Networks
#'
#' This function calculates the enrichment of a gene regulatory network over a
#' ChIP-Seq derived signature
#'
#' @param vobj a list, the output of the \code{'vulcan.normalize'} function
#' @param network an object of class \code{'viper::regulon'}
#' @param contrast a vector of two fields, containing the condition names
#' to be compared (1 vs 2)
#' @param annotation an optional named vector to convert gene identifiers
#' (e.g. entrez ids to gene symbols)
#' Default (NULL) won't convert gene names.
#' @param minsize integer indicating the minimum regulon size for the analysis
#' to be run. Default: 10
#'
#' @return A list of components:
#' \describe{
#' \item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
#' columns}
#' \item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
#' columns}
#' \item{rawcounts}{A matrix of raw gene counts, genes as rows, samples as
#' columns. The counts are associated to the promoter region of the gene}
#' \item{rpkms}{A matrix of RPKMs, genes as rows, samples as
#' columns. The RPKMs are associated to the promoter region of the gene}
#' \item{normalized}{A matrix of gene abundances normalized by
#' Variance-Stabilizing Transformation (VST), genes as rows, samples as
#' columns. The abundances are associated to the promoter
#' region of the gene}
#' \item{samples}{A vector of sample names and conditions}
#' \item{msviper}{a multisample virtual proteomics object from the
#' viper package}
#' \item{mrs}{A table of master regulators for a specific signature, indicating
#' their Normalized Enrichment Score (NES) and p-value}
#' }
#' @examples
#' library(vulcandata)
#' # Get an example vulcan object (generated with vulcan.import() using the
#' # dummy dataset contained in the \textit{vulcandata} package)
#' vobj<-vulcandata::vulcanexample()
#' # Annotate peaks to gene names
#' vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
#' # Normalize data for VULCAN analysis
#' vobj<-vulcan.normalize(vobj)
#' # Detect which conditions are present
#' names(vobj$samples)
#'
#' # Load an ARACNe network
#' # This is a regulon object as specified in the VIPER package, named 'network'
#' load(system.file('extdata','network.rda',package='vulcandata',mustWork=TRUE))
#' # Run VULCAN
#' # We can reduce the minimum regulon size, since in this example only one
#' # chromosome
#' # was measured, and the networks would otherwise have too few hits
#' vobj_analysis<-vulcan(vobj,network=network,contrast=c('t90','t0'),minsize=5)
#' # Visualize output using the msviper plotting function
#' plot(vobj_analysis$msviper,mrs=7)
#'
#' @export
vulcan <- function(vobj, network, contrast, annotation = NULL,
                minsize = 10) {
    tfs <- names(network)
    samples <- vobj$samples
    normalized <- vobj$normalized

    # Prepare output objects
    msvipers <- matrix(NA, ncol = 3, nrow = length(tfs))
    rownames(msvipers) <- tfs
    # Define contrast
    a <- samples[[contrast[1]]]
    b <- samples[[contrast[2]]]
    # Vulcan msviper implementation
    set.seed(1)
    signature <- viper::rowTtest(normalized[, a], normalized[, b])$statistic
    dnull <- viper::ttestNull(normalized[, a], normalized[, b], per = 1000)
    msviper <- msviper(signature, network,
                    dnull, minsize = minsize)
    # Annotate
    if (!is.null(annotation)) {
        msviper <- viper::msviperAnnot(msviper, annotation)
    }
    vobj$msviper <- msviper
    # Specific Master Regulators
    mrs <- cbind(msviper$es$nes, z2p(msviper$es$nes))
    colnames(mrs) <- c("NES", "pvalue")
    vobj$mrs <- mrs

    return(vobj)
}


#' Function to calculate pathway enrichment over a ChIP-Seq profile
#'
#' This function applies Gene Set Enrichment Analysis or Rank Enrichment
#' Analysis over a ChIP-Seq signature contained in a vulcan package object
#'
#' @param vobj a list, the output of the \code{'vulcan.annotate'} function
#' @param contrast a vector with the name of the two conditions to
#' compare. If method=='REA', contrast can be set to 'all', and Rank Enrichment
#' Analysis will be performed for every sample independently, compared to the
#' mean of the dataset.
#' @param pathways a list of vectors, one vector of gene identifiers per
#' pathway
#' @param method either 'REA' for Rank Enrichment Analysis or 'GSEA' for Gene
#' Set Enrichment Analysis
#' @param np numeric, only for GSEA, the number of permutations to build the
#' null distribution. Default is 1000
#' @return if method=='GSEA', a named vector, with pathway names as names,
#' and the normalized enrichment score of either the GSEA as value.
#' If method=='REA', a matrix, with pathway names as rows and specific
#' contrasts as columns (the method 'REA' allows for multiple contrasts to
#' be calculated at the same time)
#' @examples
#' library(vulcandata)
#' vfile<-tempfile()
#' vulcandata::vulcansheet(vfile)
#' #vobj<-vulcan.import(vfile)
#' vobj<-vulcandata::vulcanexample()
#' unlink(vfile)
#' vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
#' vobj<-vulcan.normalize(vobj)
#' # Create a dummy pathway list (in entrez ids)
#' pathways<-list(
#'    pathwayA=rownames(vobj$normalized)[1:20],
#'    pathwayB=rownames(vobj$normalized)[21:50]
#' )
#' # Which contrast groups can be queried
#' names(vobj$samples)
#' results_gsea<-vulcan.pathways(vobj,pathways,contrast=c('t90','t0'),
#' method='GSEA')
#' results_rea<-vulcan.pathways(vobj,pathways,contrast=c('all'),method='REA')
#' @export
vulcan.pathways <- function(vobj, pathways,
                            contrast = NULL,
                            method = c("GSEA", "REA"), np=1000) {
    normalized <- vobj$normalized
    samples <- vobj$samples
    allgenes <- unique(unlist(pathways))

    # Specific contrast
    if (!setequal(contrast, "all")) {
        # Define contrast
        a <- samples[[contrast[1]]]
        b <- samples[[contrast[2]]]

        # Prepare signature
        signature <- viper::rowTtest(normalized[, a], normalized[, b])$statistic
        if (is.matrix(signature)) {
            signature <- signature[, 1]
        }
        othergenes <- setdiff(allgenes, names(signature))
        gaussiannoise <- setNames(rnorm(length(othergenes),
                                        mean = 0, sd = 0.01),
                                othergenes)  # very small
        signature <- c(signature, gaussiannoise)

        # GSEA
        if (method == "GSEA") {
            gsea.pathways <- setNames(rep(0, length(pathways)), names(pathways))
            message("Running GSEA for ",
                    length(pathways), " pathways")
            pb <- txtProgressBar(0, length(pathways),
                                style = 3)
            i <- 0
            for (pname in names(pathways)) {
                p <- pathways[[pname]]
                obj <- gsea(reflist = signature,
                            set = p, method = "pareto",
                            np = np)
                gsea.pathways[pname] <- obj$nes
                setTxtProgressBar(pb, i <- i + 1)
            }
            return(gsea.pathways)
        }

        # REA
        if (method == "REA") {
            rea.pathways <- setNames(rep(0, length(pathways)), names(pathways))
            message("Running REA for ", length(pathways), " pathways")
            rea.pathways <- rea(signatures = signature, groups = pathways,
                                minsize = 1)
            return(rea.pathways)
        }
    } else {
        if (method != "REA") {
            stop("Multiple signatures supported only with method='REA'")
        }
        signatures <- t(scale(t(vobj$normalized)))
        rea.pathways <- rea(signatures = signatures,
                            groups = pathways, minsize = 1)
        return(rea.pathways)
    }
}

#' REA: Rank EnrichmeNt Analysis
#'
#' REA Calculates enrichment of groups of objects over a vector of values
#' associated to a population of objects
#'
#' @param signatures a named vector, with values as signature values
#' (e.g. logFC) and names as object names (e.g. gene symbols)
#' @param groups a list of vectors of objects (e.g. pathways)
#' @param sweights weights associated to objects in the signature.
#' If NULL (default) all objects are treated according to the signature rank
#' @param gweights weights associated to association strength between each
#' object and each group. If NULL (default) all associations are treated equally
#' @param minsize integer. Minimum size of the groups to be analyzed. Default=1
#'
#' @return A numeric vector of normalized enrichment scores
#'
#' @examples
#' signatures<-setNames(-sort(rnorm(1000)),paste0('gene',1:1000))
#' set1<-paste0('gene',sample(1:200,50))
#' set2<-paste0('gene',sample(1:1000,50))
#' groups<-list(set1=set1,set2=set2)
#' obj<-rea(signatures=signatures,groups=groups)
#' obj
#' @export
rea <- function(signatures, groups, sweights = NULL,
                gweights = NULL, minsize = 1) {
    ### Remove small groups
    groups <- groups[sapply(groups, length) >=
                        minsize]

    ### Treat single 'signature'
    if (is.null(nrow(signatures))) {
        signatures <- matrix(signatures,
                            length(signatures), 1,
                            dimnames = list(names(signatures),
                                            "sample1"))
    }


    ### Generate dummy signature weights
    if (is.null(sweights)) {
        sweights <- matrix(1, nrow = nrow(signatures),
                        ncol = ncol(signatures))
        dimnames(sweights) <- dimnames(signatures)
    }
    if (!identical(dim(signatures), dim(sweights))) {
        stop("Signatures and Signature weights must be
    matrices of identical size")
    }

    ### Generate dummy group weights
    if (is.null(gweights)) {
        gweights <- relist(rep(1, sum(sapply(groups,
                                            length))), skeleton = groups)
    }

    ### Apply weights to group belonging
    wgroups <- gweights
    for (i in seq_len(length(wgroups))) {
        names(wgroups[[i]]) <- groups[[i]]
    }

    ### Rank-transform columns
    ranks <- apply(signatures, 2, rank, na.last = "keep")
    ### Assign a 0 to signature weights where
    ### the signature was NA
    sweights[is.na(ranks)] <- 0
    ### 0-1 bound ranks
    boundranks <- t(t(ranks)/(colSums(!is.na(signatures)) +
                                1))
    # Treat bound ranks as quantiles in a
    # gaussian distribution (0=-Inf, 1=+Inf)
    gaussian <- qnorm(boundranks)
    ### Deal with NAs
    gaussian[is.na(gaussian)] <- 0

    ### Apply signature weights to the
    ### normalized distribution
    gaussian <- gaussian * sweights


    ### Next, we see how each of the groups are
    ### behaving in these normalized signatures
    ### Create a boolean matrix with ngroup
    ### columns and signaturelength rows,
    ### indicating the matches
    matches <- sapply(wgroups, function(group,
                                        allElements) {
        hereMatches <- as.integer(allElements %in%
                                    names(group))
        names(hereMatches) <- allElements
        # Weigth by group belonging
        weightedMatches <- hereMatches
        weightedMatches[names(group)] <- weightedMatches[names(group)] *
            group
        return(weightedMatches)
    }, allElements = rownames(gaussian))
    # And then transpose it
    matches <- t(matches)

    # Number of matches per group
    groupmatches <- rowSums(matches)

    # Relative part of the signature that
    # matches
    relativematches <- matches/groupmatches

    # This trick will overweight massively
    # small groups with all their components
    # highly-ranked.  Extreme case is with a
    # group with one gene at the top

    # The core linear algebra operation. The
    # true magic of rea
    enrichmentScore <- relativematches %*%
        gaussian

    # Finally, every enrichment is
    # square-rooted to respect the criterion
    # of normality
    normalizedEnrichmentScore <- enrichmentScore *
        sqrt(groupmatches)

    # Return output
    return(normalizedEnrichmentScore)
}

#' z2p
#'
#' This function gives a gaussian p-value corresponding to the provided Z-score
#'
#' @param z a Z score
#' @return a p-value
#' @examples
#' z<-1.96
#' z2p(z)
#' @export
z2p <- function(z) {
    pnorm(abs(z), lower.tail = FALSE) * 2
}

#' p2z
#'
#' This function gives a gaussian Z-score corresponding to the provided p-value
#' Careful: sign is not provided
#'
#' @param p a p-value
#' @return z a Z score
#' @examples
#' p<-0.05
#' p2z(p)
#' @export
p2z <- function(p) {
    qnorm(p/2, lower.tail = FALSE)
}

#' Stouffer integration of Z scores
#'
#' This function gives a gaussian Z-score corresponding to the provided p-value
#' Careful: sign is not provided
#'
#' @param x a vector of Z scores
#' @return Z an integrated Z score
#' @examples
#' zs<-c(1,3,5,2,3)
#' stouffer(zs)
#' @export
stouffer <- function(x) {
    Z <- sum(x)/sqrt(length(x))
    return(Z)
}

#' Weighted Stouffer integration of Z scores
#'
#' This function gives a gaussian Z-score corresponding to the provided p-value
#' Careful: sign is not provided
#'
#' @param x a vector of Z scores
#' @param w weight for each Z score
#' @return Z an integrated Z score
#' @examples
#' zs<-c(1,-3,5,2,3)
#' ws<-c(1,10,1,2,1)
#' wstouffer(zs,ws)
#' @export
wstouffer <- function(x, w) {
    Z <- sum(x * w)/sqrt(sum(w^2))
    return(Z)
}


#' Fisher integration of p-values
#'
#' This function applies the Fisher integration of pvalues
#'
#' @param ps a vector of p-values
#' @return p.val an integrated p-value
#' @examples
#' ps<-c(0.01,0.05,0.03,0.2)
#' fisherp(ps)
#' @export
fisherp <- function(ps) {
    Xsq <- -2 * sum(log(ps))
    p.val <- pchisq(Xsq, df = 2 * length(ps),
                    lower.tail = FALSE)
    # p<-c(Xsq = Xsq, p.value = p.val)
    return(p.val)
}


#' Slice
#'
#' This function prints a slice of a matrix
#'
#' @param matrix A matrix
#' @return prints it
#' @examples
#' set.seed(1)
#' example<-matrix(rnorm(1000),nrow=100,ncol=10)
#' slice(example)
#' @export
slice <- function(matrix) {
    if (nrow(matrix) < 5) {
        stop("Input matrix has less than 5 rows")
    }
    if (ncol(matrix) < 5) {
        stop("Input matrix has less than 5 columns")
    }
    print(matrix[1:5, 1:5])
}



#' Convert correlation coefficient to p-value
#'
#' This functions converts an R value from a correlation calculation into a
#' p-value, using a T distribution with the provided number of samples N minus
#' 2 degrees of freedom
#' @param r a correlation coefficient
#' @param N a number of samples
#' @return p a p-value
#' @examples
#' set.seed(1)
#' a<-rnorm(1000)
#' b<-a+rnorm(1000,sd=10)
#' r<-cor(a,b,method='pearson')
#' corr2p(r,N=length(a))
#' @export
corr2p <- function(r, N) {
    # Get the t-distribution value
    t <- r/sqrt((1 - r^2)/(N - 2))
    # t follows the t distribution with
    # df=N?2
    p <- 1 - pt(abs(t), N - 2)
    return(p)
}

#' Convert p-value to correlation coefficient
#'
#' This functions converts an p-value into a the corresponding correlation
#' coefficient, using a T distribution with the provided number of samples N
#' minus 2 degrees of freedom
#' @param p a p-value
#' @param N a number of samples
#' @return r a correlation coefficient
#' @examples
#' N<-100
#' p<-0.05
#' p2corr(p,N)
#' @export
p2corr <- function(p, N) {
    # Get the t-value
    t <- abs(qt(p, N - 2))

    # Convert to correlation
    r <- sqrt(t^2/(N - 2 + t^2))

    return(r)
}


#' Convert a numeric vector into colors
#' @param z a vector of numbers
#' @param col1 a color name for the min value, default 'navy'
#' @param col2 a color name for the middle value, default 'white'
#' @param col3 a color name for the max value, default 'red3'
#' @param nbreaks Number of colors to be generated. Default is 30.
#' @param center boolean, should the data be centered? Default is TRUE
#' @param rank boolean, should the data be ranked? Default is FALSE
#' @return a vector of colors
#' @examples
#' a<-rnorm(1000)
#' cols<-val2col(a)
#' plot(a,col=cols,pch=16)
#' @export
val2col <- function(z, col1 = "navy", col2 = "white",
                    col3 = "red3", nbreaks = 100, center = TRUE,
                    rank = FALSE) {
    isMatrix <- FALSE
    if (is.matrix(z)) {
        isMatrix <- TRUE
        oriz <- z
    }
    if (is.character(z)) {
        z <- as.numeric(as.factor(z))
    }
    if (rank) {
        z <- rank(z)
    }
    if (center) {
        extreme = round(max(abs(z)))
        breaks <- seq(-extreme, extreme, length = nbreaks)
        z <- z - mean(z)
    } else {
        breaks <- seq(min(z), max(z), length = nbreaks)
    }
    ncol <- length(breaks) - 1
    col <- gplots::colorpanel(ncol, col1, col2, col3)
    CUT <- cut(z, breaks = breaks)
    # assign colors to heights for each point
    colorlevels <- col[match(CUT, levels(CUT))]
    names(colorlevels) <- names(z)
    if (isMatrix) {
        colormatrix <- matrix(colorlevels, ncol = ncol(oriz), nrow = nrow(oriz))
        dimnames(colormatrix) <- dimnames(oriz)
        return(colormatrix)
    }
    return(colorlevels)
}

#' kmgformat - Nice Formatting of Numbers
#'
#' This function will convert thousand numbers to K, millions to M, billions
#' to G, trillions to T, quadrillions to P
#'
#' @param input A vector of values
#' @param roundParam How many decimal digits you want
#' @return A character vector of formatted numebr names
#' @examples
#' # Thousands
#' set.seed(1)
#' a<-runif(1000,0,1e4)
#' plot(a,yaxt='n')
#' kmg<-kmgformat(pretty(a))
#' axis(2,at=pretty(a),labels=kmg)
#'
#' # Millions to Billions
#' set.seed(1)
#' a<-runif(1000,0,1e9)
#' plot(a,yaxt='n',pch=20,col=val2col(a))
#' kmg<-kmgformat(pretty(a))
#' axis(2,at=pretty(a),labels=kmg)
#' @export
kmgformat <- function(input, roundParam = 1) {
    signs <- sign(input)
    signs[signs == 1] <- ""
    signs[signs == -1] <- "-"
    absinput <- abs(input)
    output <- c()
    for (i in absinput) {
        if (i < 1000) {
            output <- c(output, i)
        } else if (i < 1e+06) {
            i <- round(i/1000, roundParam)
            i <- paste0(i, "K")
            output <- c(output, i)
        } else if (i < 1e+09) {
            i <- round(i/1e+06, roundParam)
            i <- paste0(i, "M")
            output <- c(output, i)
        } else if (i < 1e+12) {
            i <- round(i/1e+09, roundParam)
            i <- paste0(i, "G")
            output <- c(output, i)
        } else if (i < 1e+15) {
            i <- round(i/1e+12, roundParam)
            i <- paste0(i, "T")
            output <- c(output, i)
        } else if (i < 1e+18) {
            i <- round(i/1e+15, roundParam)
            i <- paste0(i, "P")
            output <- c(output, i)
        } else {
            output <- c(output, i)
        }
    }
    output <- paste0(signs, output)
    return(output)
}



#' densityauc - Calculate the AUC of a density object
#'
#' This function will calculate the AUC of a density object generated by the
#' \code{'density'} function.
#'
#' @param dens a density object
#' @param window a vector with two values, specifying the left and right borders
#' for the AUC to be calculated
#' @return a numeric value for the density AUC
#' @examples
#' set.seed(1)
#' a<-rnorm(1000)
#' d<-density(a)
#' window<-c(2,3)
#' da<-densityauc(d,window)
#'
#' plot(d,main='')
#' abline(v=window,lty=2)
#' title(paste0('AUC between lines=',da))
#'
#'
#'
#' @export
densityauc <- function(dens, window) {
    xt <- diff(dens$x[dens$x > window[1] & dens$x < window[2]])
    yt <- rollmean(dens$y[dens$x > window[1] & dens$x < window[2]], 2)
    sum(xt * yt)
}



#' textplot2 - An x y plot of non-overlapping text
#'
#' This function is an extension of the \code{'textplot'} function from the
#' \code{'wordcloud'} package, with the extra functionality of specifiying the
#' color of the points
#' @param x x coordinates
#' @param y y coordinates
#' @param words the text to plot
#' @param cex font size
#' @param new should a new plot be created
#' @param pointcolor a string specifying the color of the points
#' (default #FFFFFF00)
#' @param show.lines if true, then lines are plotted between x,y and the word,
#' for those words not covering their x,y coordinates
#' @param pch pch parameter for the plotted points
#' @param ...    Additional parameters to be passed to wordlayout and text.
#' @return nothing
#' @examples
#' obj_names<-apply(expand.grid(LETTERS,LETTERS),1,paste,collapse='')
#' a<-setNames(runif(26*26),obj_names)
#' b<-setNames(rnorm(26*26),obj_names)
#' plot(a,b,pch=20,col='grey')
#' top<-names(sort(-a))[1:50]
#' textplot2(a[top],b[top],words=top,new=FALSE,pointcolor='black')
#' @export
textplot2 <- function(x, y, words, cex = 1,
        pch = 16, pointcolor = "#FFFFFF00", new = TRUE,
        show.lines = TRUE, ...) {
    if (new) {
        plot(x, y, type = "n", ...)
    }
    lay <- wordlayout(x, y, words, cex, ...)
    if (show.lines) {
        for (i in seq_len(length(x))) {
            xl <- lay[i, 1]
            yl <- lay[i, 2]
            w <- lay[i, 3]
            h <- lay[i, 4]
            if (x[i] < xl || x[i] > xl + w || y[i] < yl || y[i] >
                yl + h) {
                points(x[i], y[i], pch = pch, col = pointcolor, cex = 0.5)
                nx <- xl + 0.5 * w
                ny <- yl + 0.5 * h
                lines(c(x[i], nx), c(y[i], ny), col = "grey")
            }
        }
    }
    text(lay[, 1] + 0.5 * lay[, 3], lay[,
        2] + 0.5 * lay[, 4], words, cex = cex,
        ...)
}

#' Define the average fragment length
#'
#' A function to get average fragment length from a ChIP-Seq experiment
#' @param bam.files a vector of BAM files locations
#' @param plot logical. Should a plot be generated?
#' @param max.dist numeric. Maximum fragment length accepted. Default=550
#' @return nothing
#' @examples
#' library(vulcandata)
#' sheetfile<-'deleteme.csv'
#' vulcandata::vulcansheet(sheetfile)
#' a<-read.csv(sheetfile,as.is=TRUE)
#' bams<-a$bamReads
#' unlink(sheetfile)
#' average_fragment_length(bams,plot=TRUE)
#' @export
average_fragment_length <- function(bam.files,
    plot = TRUE, max.dist = 550) {
    # obtain average fragment length from
    # cross-correlation of plus/minus strand
    # alignments
    x <- csaw::correlateReads(bam.files, max.dist = max.dist)
    # visualize (raw and smoothed)
    xs <- caTools::runmean(Rle(x), k = 101, endrule = "constant")
    frag.len <- which.max(xs)
    if (plot) {
        plot(0:max.dist, x, pch = "*", ylab = "CCF", xlab = "Delay (bp)")
        lines(0:max.dist, xs, col = "red", lwd = 3)
        abline(v = frag.len, col = "blue")
        legend("topright", paste0("fragment-length = ", frag.len,
            "bp"), bty = "n")
    }
    return(frag.len)
}

%\VignetteIndexEntry{Vulcan: VirtUaL ChIP-Seq Analysis through Networks}
%\VignettePackage{vulcan}
%\VignetteEngine{utils::Sweave}

\documentclass{article}
\usepackage{fullpage}
\usepackage{hyperref}
\usepackage{authblk}
\usepackage[utf8]{inputenc}

\title{Using vulcan, a package that combines regulatory networks and ChIP-Seq
data to identify chromatin-binding cofactors}
\author[1]{Federico M. Giorgi}
\author[2]{Andrew N. Holding}
\author[3]{Florian Markowetz}
\affil[1]{Department of Pharmacy and Biotechnology, University of Bologna, Italy}
\affil[2]{Department of Biology, University of York, United Kingdom}
\affil[3]{Cancer Research UK, Cambridge Institute, United Kingdom}
\date{\today}

\begin{document}
\SweaveOpts{concordance=TRUE}
\maketitle

%-----------
\section{Overview of VULCAN}\label{sec:overview}
Vulcan (VirtUaL ChIP-Seq Analysis through Networks) is a pipeline that combines
ChIP-Seq data and regulatory networks to obtain transcription factors that are
likely affected by a specific stimulus. In order to do so, our package combines
strategies from different BioConductor packages: \emph{DESeq} for data
normalization, \emph{ChIPpeakAnno} and \emph{DiffBind} for annotation and
definition of ChIP-Seq genomic peaks, \emph{csaw} to define optimal peak width
and \emph{viper} for applying a regulatory network over a differential binding
signature (Holding et al., 2019).
Usage of gene regulatory networks to analyze biological systems has witnessed
an exponential increase in the last decade, due to the ease of obtaining
genome-wide expression data (Margolin et al., 2005; Giorgi et al., 2013; Castro
et al., 2015). Recently, the VIPER approach to interrogate these network models
has been proposed to infer transcription factor activity using the expression
of a collection of their putative targets, i.e. their regulon (Alvarez et al.,
2016). In the VIPER algorithm, gene-level differential expression signatures
are obtained for either individual samples (relative to the mean of the dataset)
or between groups of samples, and regulons are tested for enrichment. Ideally,
TF regulons can be tested on promoter-level differential binding signatures
generated from ChIP-Seq experiments, in order to ascertain the global change
in promoter occupancy for gene sets. In our study, we propose an extension
of the VIPER algorithm to specifically analyze TF occupancy in ChIP-Seq
experiments. Our VULCAN algorithm uses ChIP-Seq data obtained for a given TF
to provide candidate coregulators of the response to a given stimulus.
The analysis is based on identifying differentially bound genes and testing
their enrichment in the regulon of potential co-regulatory factors.

%--------
\section{Citation}
Holding, A.N., Giorgi, F.M., Donnelly, A., Cullen, A.E., Nagarajan, S., Selth,
L.A. and Markowetz, F., 2019. VULCAN integrates ChIP-seq with patient-derived
co-expression networks to identify GRHL2 as a key co-regulator of ERa at
enhancers in breast cancer. Genome biology, 20(1), p.91.
%--------
\section{Installation of \emph{vulcan} package}
VULCAN requires the R-system (\url{http://www.r-project.org}), and the
\emph{vulcandata} package to run the examples.
After installing R, all required components can be obtained with:

<<echo=FALSE, results=hide>>=
set.seed(1)
@

<<echo=TRUE, results=hide, eval=FALSE>>=
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("vulcandata")
BiocManager::install("vulcan")
@

%----
\section{Getting started}
As first step, we have to load the vulcan package with:
<<echo=TRUE, results=hide>>=
library(vulcan)
@
Only to run the examples, a set of reduced BAM files can be accessed by loading
the \emph{vulcandata} package:
<<echo=TRUE, results=hide>>=
library(vulcandata)
@


%-------
\section{Running an example VULCAN pipeline}
\subsection{Generate a example annotation file}
<<echo=TRUE, results=hide>>=
# Generate an annotation file from the dummy ChIP-Seq dataset
vfile<-tempfile()
vulcandata::vulcansheet(vfile)
@

\subsection{Import ChIP-Seq information}
<<echo=TRUE, results=hide>>=
# Import BAM and BED information into a list object
# vobj<-vulcan.import(vfile)
vobj<-vulcandata::vulcanexample()
unlink(vfile)
@

\subsection{Annotate peaks}
<<echo=TRUE, results=hide>>=
# Annotate peaks to gene names
vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method="sum")
@

\subsection{Peak abundance normalization}
<<echo=TRUE, results=hide>>=
# Normalize data for VULCAN analysis
vobj<-vulcan.normalize(vobj)
# Detect which conditions are present in our imported object
names(vobj$samples)
@

\subsection{Load an ARACNe network}
We will use a regulon object as specified in the \emph{viper} package, named
"network"
<<echo=TRUE, results=hide>>=
load(system.file("extdata","network.rda",package="vulcandata",mustWork=TRUE))
@

\subsection{Run VULCAN}
Once we have prepared 1) the ChIP-Seq annotated and normalized object and 2)
a gene regulatory network, we can run the VULCAN pipeline.
We can reduce the minimum regulon size, since in the \emph{vulcandata} example
only one chromosome was used, and the networks would otherwise have too few
hits. We can then visualize the output using the msviper plotting function
<<echo=TRUE, results=hide, fig=TRUE>>=
vobj_analysis<-vulcan(vobj,network=network,contrast=c("t90","t0"),minsize=5)
plot(vobj_analysis$msviper,mrs=7)
@


%-------
\section{More convenience tools for ChIP-Seq functional data analysis}

\subsection{Gene Set Enrichment Analysis}
VULCAN contains a GSEA (Subramanian et al., 2005) implementation that can fit
a Pareto tail distribution for p-value estimation. In the following example,
we generate an artificial signature of 1000 genes with normally-distributed
positive expression. The gene set that is evaluated is composed by 50 genes
randomly picked from one of the tails
<<echo=TRUE, results=hide>>=
reflist<-setNames(-sort(rnorm(1000)),paste0("gene",1:1000))
set<-paste0("gene",sample(1:200,50))
obj<-gsea(reflist,set,method="pareto")
obj$p.value
@
The GSEA object can be plotted
<<echo=TRUE, results=hide, fig=TRUE>>=
plot_gsea(obj)
@

\subsection{Rank Enrichment Analysis}
Rank Enrichment Analysis (REA Calculates) enrichment of groups of objects over
a vector of values associated to a population of objects using a linear
algebraic matrix multiplication.
<<echo=TRUE, results=hide>>=
signatures<-setNames(-sort(rnorm(1000)),paste0("gene",1:1000))
set1<-paste0("gene",sample(1:200,50))
set2<-paste0("gene",sample(1:1000,50))
groups<-list(set1=set1,set2=set2)
obj<-rea(signatures=signatures,groups=groups)
obj
@


\subsection{Convenience functions}
The function \emph{kmgformat} will convert thousand numbers to K, millions to M,
billions to G, trillions to T, quadrillions to P. The function \emph{val2col}
converts a distribution of numbers to a distribution of colors.
<<echo=TRUE, results=hide, fig=TRUE>>=
par(mfrow=c(1,2))
# Thousands
set.seed(1)
a<-runif(1000,0,1e4)
plot(a,yaxt="n")
kmg<-kmgformat(pretty(a))
axis(2,at=pretty(a),labels=kmg)

# Millions to Billions
set.seed(1)
a<-runif(1000,0,1e9)
plot(a,yaxt="n",pch=20,col=val2col(a))
kmg<-kmgformat(pretty(a))
axis(2,at=pretty(a),labels=kmg)
@


%-----------
\begin{thebibliography}{99}
\bibitem{Holding2019} Holding, A.N., Giorgi, F.M., Donnelly, A., Cullen, A.E.,
Nagarajan, S., Selth, L.A. \& Markowetz, F., (2019). VULCAN integrates ChIP-seq
with patient-derived co-expression networks to identify GRHL2 as a key
co-regulator of ERa at enhancers in breast cancer. Genome biology, 20(1), p.91.
\bibitem{Alvarez2013} Alvarez, M.J., Shen, Y., Giorgi, F.M., Lachmann, A.,
Ding, B.B., Ye, B.H., \& Califano, A. (2016). Functional characterization
of somatic mutations in cancer using network-based inference of protein
activity. Nature Genetics.
\bibitem{Margolin2005} Margolin, A.A., Nemenman, I., Basso, K., Wiggins, C.,
Stolovitzky, G., Dalla Favera, R., \& Califano, A. (2006). ARACNE: an algorithm
for the reconstruction of gene regulatory networks in a mammalian cellular
context. BMC bioinformatics, 7(1), S7.
\bibitem{Castro2015} Castro, M.A., de Santiago, I., Campbell, T.M.,
Vaughn, C., Hickey, T.E., Ross, E., Tilley, W.D., Markowetz, F.,
Ponder, B.A. \& Meyer, K.B., 2015. Regulators of genetic risk of breast cancer
identified by integrative network analysis. Nature genetics.
\bibitem{Giorgi2013} Giorgi, F.M., Del Fabbro, C., \& Licausi, F. (2013).
Comparative study of RNA-seq-and microarray-derived coexpression networks in
Arabidopsis thaliana. Bioinformatics, 29(6), 717-724.
\bibitem{Subramanian2005} Subramanian, A., Tamayo, P., Mootha, V.K.,
Mukherjee, S., Ebert, B.L., Gillette, M.A. \& Mesirov, J.P. (2005).
Gene set enrichment analysis: a knowledge-based approach for interpreting
genome-wide expression profiles. Proceedings of the National Academy
of Sciences, 102(43), 15545-15550.
\end{thebibliography}
%----------
\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulcan.R
\name{vulcan.pathways}
\alias{vulcan.pathways}
\title{Function to calculate pathway enrichment over a ChIP-Seq profile}
\usage{
vulcan.pathways(
  vobj,
  pathways,
  contrast = NULL,
  method = c("GSEA", "REA"),
  np = 1000
)
}
\arguments{
\item{vobj}{a list, the output of the \code{'vulcan.annotate'} function}

\item{pathways}{a list of vectors, one vector of gene identifiers per
pathway}

\item{contrast}{a vector with the name of the two conditions to
compare. If method=='REA', contrast can be set to 'all', and Rank Enrichment
Analysis will be performed for every sample independently, compared to the
mean of the dataset.}

\item{method}{either 'REA' for Rank Enrichment Analysis or 'GSEA' for Gene
Set Enrichment Analysis}

\item{np}{numeric, only for GSEA, the number of permutations to build the
null distribution. Default is 1000}
}
\value{
if method=='GSEA', a named vector, with pathway names as names,
and the normalized enrichment score of either the GSEA as value.
If method=='REA', a matrix, with pathway names as rows and specific
contrasts as columns (the method 'REA' allows for multiple contrasts to
be calculated at the same time)
}
\description{
This function applies Gene Set Enrichment Analysis or Rank Enrichment
Analysis over a ChIP-Seq signature contained in a vulcan package object
}
\examples{
library(vulcandata)
vfile<-tempfile()
vulcandata::vulcansheet(vfile)
#vobj<-vulcan.import(vfile)
vobj<-vulcandata::vulcanexample()
unlink(vfile)
vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
vobj<-vulcan.normalize(vobj)
# Create a dummy pathway list (in entrez ids)
pathways<-list(
   pathwayA=rownames(vobj$normalized)[1:20],
   pathwayB=rownames(vobj$normalized)[21:50]
)
# Which contrast groups can be queried
names(vobj$samples)
results_gsea<-vulcan.pathways(vobj,pathways,contrast=c('t90','t0'),
method='GSEA')
results_rea<-vulcan.pathways(vobj,pathways,contrast=c('all'),method='REA')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{fisherp}
\alias{fisherp}
\title{Fisher integration of p-values}
\usage{
fisherp(ps)
}
\arguments{
\item{ps}{a vector of p-values}
}
\value{
p.val an integrated p-value
}
\description{
This function applies the Fisher integration of pvalues
}
\examples{
ps<-c(0.01,0.05,0.03,0.2)
fisherp(ps)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{kmgformat}
\alias{kmgformat}
\title{kmgformat - Nice Formatting of Numbers}
\usage{
kmgformat(input, roundParam = 1)
}
\arguments{
\item{input}{A vector of values}

\item{roundParam}{How many decimal digits you want}
}
\value{
A character vector of formatted numebr names
}
\description{
This function will convert thousand numbers to K, millions to M, billions
to G, trillions to T, quadrillions to P
}
\examples{
# Thousands
set.seed(1)
a<-runif(1000,0,1e4)
plot(a,yaxt='n')
kmg<-kmgformat(pretty(a))
axis(2,at=pretty(a),labels=kmg)

# Millions to Billions
set.seed(1)
a<-runif(1000,0,1e9)
plot(a,yaxt='n',pch=20,col=val2col(a))
kmg<-kmgformat(pretty(a))
axis(2,at=pretty(a),labels=kmg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{dpareto}
\alias{dpareto}
\title{Probability density of Pareto distributions}
\usage{
dpareto(x, threshold = 1, exponent, log = FALSE)
}
\arguments{
\item{x}{Data vector of log probability densities}

\item{threshold}{numeric value to define the start of the tail}

\item{exponent}{the exponent obtained from the pareto.fit function}

\item{log}{logical, should the values be log-transformed?}
}
\value{
Vector of (log) probability densities
}
\description{
Gives NA on values below the threshold
}
\examples{
set.seed(1)
x<-abs(rnorm(1000))
n<-length(x)
exponent<-1+n/sum(log(x))
dp<-dpareto(x,exponent=exponent)
plot(dp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulcan.R
\name{vulcan}
\alias{vulcan}
\title{VULCAN - VirtUaL Chipseq data Analysis using Networks}
\usage{
vulcan(vobj, network, contrast, annotation = NULL, minsize = 10)
}
\arguments{
\item{vobj}{a list, the output of the \code{'vulcan.normalize'} function}

\item{network}{an object of class \code{'viper::regulon'}}

\item{contrast}{a vector of two fields, containing the condition names
to be compared (1 vs 2)}

\item{annotation}{an optional named vector to convert gene identifiers
(e.g. entrez ids to gene symbols)
Default (NULL) won't convert gene names.}

\item{minsize}{integer indicating the minimum regulon size for the analysis
to be run. Default: 10}
}
\value{
A list of components:
\describe{
\item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
columns}
\item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
columns}
\item{rawcounts}{A matrix of raw gene counts, genes as rows, samples as
columns. The counts are associated to the promoter region of the gene}
\item{rpkms}{A matrix of RPKMs, genes as rows, samples as
columns. The RPKMs are associated to the promoter region of the gene}
\item{normalized}{A matrix of gene abundances normalized by
Variance-Stabilizing Transformation (VST), genes as rows, samples as
columns. The abundances are associated to the promoter
region of the gene}
\item{samples}{A vector of sample names and conditions}
\item{msviper}{a multisample virtual proteomics object from the
viper package}
\item{mrs}{A table of master regulators for a specific signature, indicating
their Normalized Enrichment Score (NES) and p-value}
}
}
\description{
This function calculates the enrichment of a gene regulatory network over a
ChIP-Seq derived signature
}
\examples{
library(vulcandata)
# Get an example vulcan object (generated with vulcan.import() using the
# dummy dataset contained in the \textit{vulcandata} package)
vobj<-vulcandata::vulcanexample()
# Annotate peaks to gene names
vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
# Normalize data for VULCAN analysis
vobj<-vulcan.normalize(vobj)
# Detect which conditions are present
names(vobj$samples)

# Load an ARACNe network
# This is a regulon object as specified in the VIPER package, named 'network'
load(system.file('extdata','network.rda',package='vulcandata',mustWork=TRUE))
# Run VULCAN
# We can reduce the minimum regulon size, since in this example only one
# chromosome
# was measured, and the networks would otherwise have too few hits
vobj_analysis<-vulcan(vobj,network=network,contrast=c('t90','t0'),minsize=5)
# Visualize output using the msviper plotting function
plot(vobj_analysis$msviper,mrs=7)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{slice}
\alias{slice}
\title{Slice}
\usage{
slice(matrix)
}
\arguments{
\item{matrix}{A matrix}
}
\value{
prints it
}
\description{
This function prints a slice of a matrix
}
\examples{
set.seed(1)
example<-matrix(rnorm(1000),nrow=100,ncol=10)
slice(example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{wstouffer}
\alias{wstouffer}
\title{Weighted Stouffer integration of Z scores}
\usage{
wstouffer(x, w)
}
\arguments{
\item{x}{a vector of Z scores}

\item{w}{weight for each Z score}
}
\value{
Z an integrated Z score
}
\description{
This function gives a gaussian Z-score corresponding to the provided p-value
Careful: sign is not provided
}
\examples{
zs<-c(1,-3,5,2,3)
ws<-c(1,10,1,2,1)
wstouffer(zs,ws)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{densityauc}
\alias{densityauc}
\title{densityauc - Calculate the AUC of a density object}
\usage{
densityauc(dens, window)
}
\arguments{
\item{dens}{a density object}

\item{window}{a vector with two values, specifying the left and right borders
for the AUC to be calculated}
}
\value{
a numeric value for the density AUC
}
\description{
This function will calculate the AUC of a density object generated by the
\code{'density'} function.
}
\examples{
set.seed(1)
a<-rnorm(1000)
d<-density(a)
window<-c(2,3)
da<-densityauc(d,window)

plot(d,main='')
abline(v=window,lty=2)
title(paste0('AUC between lines=',da))



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulcan.R
\name{vulcan.normalize}
\alias{vulcan.normalize}
\title{Function to normalize promoter peak data}
\usage{
vulcan.normalize(vobj)
}
\arguments{
\item{vobj}{a list, the output of the \code{'vulcan.annotate'} function}
}
\value{
A list of components:
\describe{
\item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
columns}
\item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
columns}
\item{rawcounts}{A matrix of raw gene counts, genes as rows, samples as
columns. The counts are associated to the promoter region of the gene}
\item{rpkms}{A matrix of RPKMs, genes as rows, samples as
columns. The RPKMs are associated to the promoter region of the gene}
\item{normalized}{A matrix of gene abundances normalized by
Variance-Stabilizing Transformation (VST), genes as rows, samples as
columns. The abundances are associated to the promoter
region of the gene}
\item{samples}{A vector of sample names and conditions}
}
}
\description{
This function normalizes gene-centered ChIP-Seq data using VST
}
\examples{
\dontrun{
library(vulcandata)
vobj<-vulcandata::vulcanexample()
vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
vobj<-vulcan.normalize(vobj)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{p2corr}
\alias{p2corr}
\title{Convert p-value to correlation coefficient}
\usage{
p2corr(p, N)
}
\arguments{
\item{p}{a p-value}

\item{N}{a number of samples}
}
\value{
r a correlation coefficient
}
\description{
This functions converts an p-value into a the corresponding correlation
coefficient, using a T distribution with the provided number of samples N
minus 2 degrees of freedom
}
\examples{
N<-100
p<-0.05
p2corr(p,N)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulcan.R
\name{vulcan.import}
\alias{vulcan.import}
\title{Function to import BAM files}
\usage{
vulcan.import(sheetfile, intervals = NULL)
}
\arguments{
\item{sheetfile}{path to a csv annotation file containing sample information
and BAM location}

\item{intervals}{size of the peaks. If NULL (default) it is inferred from
the average fragment length observed in the dataset}
}
\value{
A list of components:
\describe{
\item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
columns}
\item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
columns}
\item{samples}{A vector of sample names and conditions}
}
}
\description{
This function coalesces and annotates a set of BAM files into peak-centered
data
}
\examples{
library(vulcandata)
# Generate an annotation file from the dummy ChIP-Seq dataset
vfile<-tempfile()
vulcandata::vulcansheet(vfile)
# Import BAM and BED information into a list object
# vobj<-vulcan.import(vfile)
# This vobj is identical to the object returned by
# vulcandata::vulcanexample()
unlink(vfile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{average_fragment_length}
\alias{average_fragment_length}
\title{Define the average fragment length}
\usage{
average_fragment_length(bam.files, plot = TRUE, max.dist = 550)
}
\arguments{
\item{bam.files}{a vector of BAM files locations}

\item{plot}{logical. Should a plot be generated?}

\item{max.dist}{numeric. Maximum fragment length accepted. Default=550}
}
\value{
nothing
}
\description{
A function to get average fragment length from a ChIP-Seq experiment
}
\examples{
library(vulcandata)
sheetfile<-'deleteme.csv'
vulcandata::vulcansheet(sheetfile)
a<-read.csv(sheetfile,as.is=TRUE)
bams<-a$bamReads
unlink(sheetfile)
average_fragment_length(bams,plot=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{textplot2}
\alias{textplot2}
\title{textplot2 - An x y plot of non-overlapping text}
\usage{
textplot2(
  x,
  y,
  words,
  cex = 1,
  pch = 16,
  pointcolor = "#FFFFFF00",
  new = TRUE,
  show.lines = TRUE,
  ...
)
}
\arguments{
\item{x}{x coordinates}

\item{y}{y coordinates}

\item{words}{the text to plot}

\item{cex}{font size}

\item{pch}{pch parameter for the plotted points}

\item{pointcolor}{a string specifying the color of the points
(default #FFFFFF00)}

\item{new}{should a new plot be created}

\item{show.lines}{if true, then lines are plotted between x,y and the word,
for those words not covering their x,y coordinates}

\item{...}{Additional parameters to be passed to wordlayout and text.}
}
\value{
nothing
}
\description{
This function is an extension of the \code{'textplot'} function from the
\code{'wordcloud'} package, with the extra functionality of specifiying the
color of the points
}
\examples{
obj_names<-apply(expand.grid(LETTERS,LETTERS),1,paste,collapse='')
a<-setNames(runif(26*26),obj_names)
b<-setNames(rnorm(26*26),obj_names)
plot(a,b,pch=20,col='grey')
top<-names(sort(-a))[1:50]
textplot2(a[top],b[top],words=top,new=FALSE,pointcolor='black')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{gsea}
\alias{gsea}
\title{GSEA}
\usage{
gsea(
  reflist,
  set,
  method = c("permutation", "pareto"),
  np = 1000,
  w = 1,
  gsea_null = NULL
)
}
\arguments{
\item{reflist}{named vector of reference scores}

\item{set}{element set}

\item{method}{one of 'permutation' or 'pareto'}

\item{np}{Number of permutations (Default: 1000)}

\item{w}{exponent used to raise the supplied scores. Default is 1 (original
scores unchanged)}

\item{gsea_null}{a GSEA null distribution (Optional)}
}
\value{
A GSEA object. Basically a list of s components:
\describe{
\item{ES}{The enrichment score}
\item{NES}{The normalized enrichment socre}
\item{ledge}{The items in the leading edge}
\item{p.value}{The permutation-based p-value}
}
}
\description{
This function performs Gene Set Enrichment Analysis
}
\examples{
reflist<-setNames(-sort(rnorm(1000)),paste0('gene',1:1000))
set<-paste0('gene',sample(1:200,50))
obj<-gsea(reflist,set,method='pareto',np=1000)
obj$p.value
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rea.R
\name{rea}
\alias{rea}
\title{REA: Rank EnrichmeNt Analysis}
\usage{
rea(signatures, groups, sweights = NULL, gweights = NULL, minsize = 1)
}
\arguments{
\item{signatures}{a named vector, with values as signature values
(e.g. logFC) and names as object names (e.g. gene symbols)}

\item{groups}{a list of vectors of objects (e.g. pathways)}

\item{sweights}{weights associated to objects in the signature.
If NULL (default) all objects are treated according to the signature rank}

\item{gweights}{weights associated to association strength between each
object and each group. If NULL (default) all associations are treated equally}

\item{minsize}{integer. Minimum size of the groups to be analyzed. Default=1}
}
\value{
A numeric vector of normalized enrichment scores
}
\description{
REA Calculates enrichment of groups of objects over a vector of values
associated to a population of objects
}
\examples{
signatures<-setNames(-sort(rnorm(1000)),paste0('gene',1:1000))
set1<-paste0('gene',sample(1:200,50))
set2<-paste0('gene',sample(1:1000,50))
groups<-list(set1=set1,set2=set2)
obj<-rea(signatures=signatures,groups=groups)
obj
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{p2z}
\alias{p2z}
\title{p2z}
\usage{
p2z(p)
}
\arguments{
\item{p}{a p-value}
}
\value{
z a Z score
}
\description{
This function gives a gaussian Z-score corresponding to the provided p-value
Careful: sign is not provided
}
\examples{
p<-0.05
p2z(p)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{val2col}
\alias{val2col}
\title{Convert a numeric vector into colors}
\usage{
val2col(
  z,
  col1 = "navy",
  col2 = "white",
  col3 = "red3",
  nbreaks = 100,
  center = TRUE,
  rank = FALSE
)
}
\arguments{
\item{z}{a vector of numbers}

\item{col1}{a color name for the min value, default 'navy'}

\item{col2}{a color name for the middle value, default 'white'}

\item{col3}{a color name for the max value, default 'red3'}

\item{nbreaks}{Number of colors to be generated. Default is 30.}

\item{center}{boolean, should the data be centered? Default is TRUE}

\item{rank}{boolean, should the data be ranked? Default is FALSE}
}
\value{
a vector of colors
}
\description{
Convert a numeric vector into colors
}
\examples{
a<-rnorm(1000)
cols<-val2col(a)
plot(a,col=cols,pch=16)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{pareto.fit}
\alias{pareto.fit}
\title{Estimate parameters of Pareto distribution}
\usage{
pareto.fit(data, threshold)
}
\arguments{
\item{data}{data vector, lower threshold (or 'find', indicating it should be
found from the data), method (likelihood or regression, defaulting to former)}

\item{threshold}{numeric value to define the start of the tail}
}
\value{
List indicating type of distribution ('pareto'), parameters,
information about fit (depending on method), OR a warning and NA if method
is not recognized
}
\description{
A wrapper for functions implementing actual methods
}
\examples{
# Estimate the tail of a pospulation normally distributed
set.seed(1)
x<-rnorm(1000)
q95<-as.numeric(quantile(abs(x),0.95))
fit<-pareto.fit(abs(x),threshold=q95)
# We can infer the pvalue of a value very much right to the tail of the
# distribution
value<-5
pvalue<-ppareto(value, threshold=q95, exponent=fit$exponent,
lower.tail=FALSE)/20
plot(density(abs(x)),xlim=c(0,value+0.3),main='Pareto fit')
arrows(value,0.2,value,0)
text(value,0.2,labels=paste0('p=',signif(pvalue,2)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{z2p}
\alias{z2p}
\title{z2p}
\usage{
z2p(z)
}
\arguments{
\item{z}{a Z score}
}
\value{
a p-value
}
\description{
This function gives a gaussian p-value corresponding to the provided Z-score
}
\examples{
z<-1.96
z2p(z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{corr2p}
\alias{corr2p}
\title{Convert correlation coefficient to p-value}
\usage{
corr2p(r, N)
}
\arguments{
\item{r}{a correlation coefficient}

\item{N}{a number of samples}
}
\value{
p a p-value
}
\description{
This functions converts an R value from a correlation calculation into a
p-value, using a T distribution with the provided number of samples N minus
2 degrees of freedom
}
\examples{
set.seed(1)
a<-rnorm(1000)
b<-a+rnorm(1000,sd=10)
r<-cor(a,b,method='pearson')
corr2p(r,N=length(a))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{plot_gsea}
\alias{plot_gsea}
\title{Plot GSEA results}
\usage{
plot_gsea(
  gsea.obj,
  twoColors = c("red", "blue"),
  plotNames = FALSE,
  colBarcode = "black",
  title = "Running Enrichment Score",
  bottomYtitle = "List Values",
  bottomYlabel = "Signature values",
  ext_nes = NULL,
  omit_middle = FALSE
)
}
\arguments{
\item{gsea.obj}{GSEA object produced by the \code{gsea} function}

\item{twoColors}{the two colors to use for positive[1] and negative[2]
enrichment scores}

\item{plotNames}{Logical. Should the set names be plotted?}

\item{colBarcode}{The color of the barcode}

\item{title}{String to be plotted above the Running Enrichment Score}

\item{bottomYtitle}{String for the title of the bottom part of the plot}

\item{bottomYlabel}{String for the label}

\item{ext_nes}{Provide a NES from an external calculation}

\item{omit_middle}{If TRUE, will not plot the running score
(FALSE by default)}
}
\value{
Nothing, a plot is generated in the default output device
}
\description{
This function generates a GSEA plot from a gsea object
}
\examples{
reflist<-setNames(-sort(rnorm(26)),LETTERS)
set<-c('A','B','D','F')
obj<-gsea(reflist,set,method='pareto')
plot_gsea(obj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vulcan.R
\name{vulcan.annotate}
\alias{vulcan.annotate}
\title{Function to annotate peaks for VULCAN analysis}
\usage{
vulcan.annotate(
  vobj,
  lborder = -10000,
  rborder = 10000,
  method = c("closest", "strongest", "sum", "topvar", "farthest", "lowvar"),
  TxDb = NULL
)
}
\arguments{
\item{vobj}{A list of peakcounts, samples and peakrpkms (i.e. the output of
the function vulcan.import)}

\item{lborder}{Boundary for peak annotation (in nucleotides) upstream of
the Transcription starting site (default: -10000)}

\item{rborder}{Boundary for peak annotation (in nucleotides) downstream of
the Transcription starting site (default: 10000)}

\item{method}{Method to deal with multiple peaks found within gene promoter
boundaries. One of sum (default), closest, strongest, topvar, farthest or
lowvar. This will affect only genes with multiple possible peaks. When a
single peak can be mapped to the promoter region of the gene, that peak
abundance will be considered as the gene promoter's occupancy.
\describe{
\item{sum}{when multiple peaks are found, sum their contributions}
\item{closest}{when multiple peaks are found, keep only the closest to the
TSS as the representative one}
\item{strongest}{when multiple peaks are found, keep the strongest as the
representative one}
\item{farthest}{when multiple peaks are found, keep only the closest to the
TSS as the representative one}
\item{topvar}{when multiple peaks are found, keep the most varying as the
representative one}
\item{lowvar}{when multiple peaks are found, keep the least varying as the
representative one}
}}

\item{TxDb}{TxDb annotation object containing the knownGene track. If NULL
(the default), TxDb.Hsapiens.UCSC.hg19.knownGene is loaded}
}
\value{
A list of components:
\describe{
\item{peakcounts}{A matrix of raw peak counts, peaks as rows, samples as
columns}
\item{peakrpkms}{A matrix of peak RPKMs, peaks as rows, samples as
columns}
\item{rawcounts}{A matrix of raw gene counts, genes as rows, samples as
columns. The counts are associated to the promoter region of the gene}
\item{rpkms}{A matrix of RPKMs, genes as rows, samples as
columns. The RPKMs are associated to the promoter region of the gene}
\item{samples}{A vector of sample names and conditions}
}
}
\description{
This function coalesces and annotates a set of BAM files into peak-centered
data. It implements the ChIPPeakANno methods, with specific choices dealing
with defining the genomic area around the promoter and which peaks to
include.
}
\examples{
library(vulcandata)
vobj<-vulcandata::vulcanexample()
vobj<-vulcan.annotate(vobj,lborder=-10000,rborder=10000,method='sum')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{ppareto}
\alias{ppareto}
\title{Cumulative distribution function of the Pareto distributions
'
Gives NA on values < threshold}
\usage{
ppareto(x, threshold = 1, exponent, lower.tail = TRUE)
}
\arguments{
\item{x}{Data vector, lower threshold, scaling exponent, usual flags}

\item{threshold}{numeric value to define the start of the tail}

\item{exponent}{the exponent obtained from the pareto.fit function}

\item{lower.tail}{logical. If the lower tail of the distribution should be
considered. Default is TRUE}
}
\value{
Vector of (log) probabilities
}
\description{
Cumulative distribution function of the Pareto distributions
'
Gives NA on values < threshold
}
\examples{
# Estimate the tail of a pospulation normally distributed
set.seed(1)
x<-rnorm(1000)
q95<-as.numeric(quantile(abs(x),0.95))
fit<-pareto.fit(abs(x),threshold=q95)
# We can infer the pvalue of a value very much right to the tail of the
# distribution
value<-5
pvalue<-ppareto(value, threshold=q95, exponent=fit$exponent,
lower.tail=FALSE)/20
plot(density(abs(x)),xlim=c(0,value+0.3),main='Pareto fit')
arrows(value,0.2,value,0)
text(value,0.2,labels=paste0('p=',signif(pvalue,2)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generic.R
\name{stouffer}
\alias{stouffer}
\title{Stouffer integration of Z scores}
\usage{
stouffer(x)
}
\arguments{
\item{x}{a vector of Z scores}
}
\value{
Z an integrated Z score
}
\description{
This function gives a gaussian Z-score corresponding to the provided p-value
Careful: sign is not provided
}
\examples{
zs<-c(1,3,5,2,3)
stouffer(zs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{null_gsea}
\alias{null_gsea}
\title{Calculate Null Distribution for GSEA}
\usage{
null_gsea(set, reflist, w = 1, np = 1000)
}
\arguments{
\item{set}{A vector containing gene names.}

\item{reflist}{A named vector containing the weights of the entire signature}

\item{w}{exponent used to raise the supplied scores. Default is 1 (original
scores unchanged)}

\item{np}{Number of permutations (Default: 1000)}
}
\value{
A vector of null scores appropriate for the set/reflist combination
provided
}
\description{
This function generates a GSEA null distribution from
}
\examples{
reflist<-setNames(-sort(rnorm(26)),LETTERS)
set<-c('A','B','D','F')
nulldist<-null_gsea(set,reflist)
nulldist[1:10]
}
